package epsilonStandalone;

import com.google.common.base.Objects;
import epsilonStandalone.ExperimentModel;
import epsilonStandalone.SimQpnResultsParser;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.eclipse.epsilon.common.parse.problem.ParseProblem;
import org.eclipse.epsilon.eol.IEolModule;
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException;
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException;
import org.eclipse.epsilon.eol.execute.context.Variable;
import org.eclipse.epsilon.eol.models.IModel;
import org.eclipse.epsilon.eol.types.EolPrimitiveType;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class DNIRunner {
  protected String simqpn = "bash /home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/simutools2016/results/runSimQPN.sh -r simutools2016 ";
  
  protected boolean simulationSilent = true;
  
  protected int kiloMultipler = 1000;
  
  protected String pwdUnix = (System.getProperty("user.dir") + "/");
  
  protected String pwdURI = ("file:" + this.pwdUnix);
  
  protected String dniModelsDirRelative = "src/epsilonStandalone/models/dni/";
  
  protected String tmpModelsDirRelative = "src/epsilonStandalone/models/tmp/";
  
  protected String qpmeModelsDirRelative = "src/epsilonStandalone/models/qpme/";
  
  protected String qpmeResultsDirRelative = "src/epsilonStandalone/models/qpmeResults/";
  
  protected String dni_base_model_file = "models/scenario0A.model";
  
  protected final LinkedList<Object> dniModels = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeModels = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeFiles = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeSimulationResultFiles = CollectionLiterals.<Object>newLinkedList();
  
  protected final LinkedList<Object> qpmeSimulationProcessedResults = CollectionLiterals.<Object>newLinkedList();
  
  protected SimQpnResultsParser simQpnParser = new SimQpnResultsParser();
  
  protected HashMap<String, List<Variable>> epsilonTransformationParameters = new HashMap<String, List<Variable>>();
  
  protected String currentTransformationFile = "etl/standalone-DNI-QPN-new.etl";
  
  public static void main(final String[] args) {
    new DNIRunner().start_single();
  }
  
  public void start_single() {
    this.simQpnParser.interestingPlaces = CollectionLiterals.<String>newLinkedList("SW35");
    this.simQpnParser.interestingColors = CollectionLiterals.<String>newLinkedList("Traffic");
    this.simQpnParser.interestingMetrics = CollectionLiterals.<String>newLinkedList("arrivThrPut");
    this.epsilonTransformationParameters.clear();
    String expNamePrefix = "AA_Debug_run";
    String experimentName = expNamePrefix;
    ExperimentModel pairListExpNameDniModel = null;
    pairListExpNameDniModel = this.stateless_applyEolStriptToBaseModel(this.dni_base_model_file, "eol/simutools_check.eol", CollectionLiterals.<Variable>newLinkedList(), 0, experimentName);
    String dniModelURI = pairListExpNameDniModel.getModelURI();
    Variable _variable = new Variable("param_goOverSdnSW", Boolean.valueOf(true), EolPrimitiveType.Boolean);
    List<Variable> paramList = CollectionLiterals.<Variable>newLinkedList(_variable);
    String outFileName = (experimentName + ".qpmemodel");
    String destinationModelURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
    InputOutput.<String>println(("Will store transformation result in: " + destinationModelURI));
    destinationModelURI = "file:models/qpme/demo.qpmemodel";
    String outputQpnURI = this.stateless_applyEtlStript("DNI", dniModelURI, "metamodels/dnimm3.ecore", 
      "QPME", destinationModelURI, "metamodels/qpme.ecore", 
      this.currentTransformationFile, paramList, experimentName);
    String qpeFileContents = this.stateless_applyEglScript("QPME", outputQpnURI, "metamodels/qpme.ecore", "egl/dni2qpme.egl", experimentName);
    String outputQpeFilePath = (((this.pwdUnix + this.qpmeModelsDirRelative) + experimentName) + ".qpe");
    this.saveOutput(qpeFileContents, outputQpeFilePath);
    InputOutput.<String>println(("Your .qpe file is here: " + outputQpeFilePath));
    this.cleanup();
  }
  
  private boolean deleteFile(final String path) {
    File f = null;
    File _file = new File(path);
    f = _file;
    return f.delete();
  }
  
  public static String getCurrentTimeStamp() {
    SimpleDateFormat sdfDate = new SimpleDateFormat("yyy.MM.dd.HH.mm.ss");
    Date now = new Date();
    String strDate = sdfDate.format(now);
    return strDate;
  }
  
  private List<String> getFileList(final String directory, final String endsWith, final String contains) {
    List<String> files = CollectionLiterals.<String>newLinkedList();
    File folder = new File(directory);
    File[] listOfFiles = folder.listFiles();
    for (final File f : listOfFiles) {
      {
        File file = ((File) f);
        boolean _isFile = file.isFile();
        if (_isFile) {
          String fname = file.getName();
          if (((fname.endsWith(endsWith) && (!endsWith.isEmpty())) && contains.isEmpty())) {
            String path = ((directory + "/") + fname);
            files.add(path);
          } else {
            if (((fname.contains(contains) && (!contains.isEmpty())) && (!contains.isEmpty()))) {
              String path_1 = ((directory + "/") + fname);
              files.add(path_1);
            } else {
              if ((fname.contains(contains) && fname.endsWith(endsWith))) {
                String path_2 = ((directory + "/") + fname);
                files.add(path_2);
              }
            }
          }
        }
      }
    }
    return files;
  }
  
  protected void parseAllResults(final String finalFileName) {
    String finalResult = "";
    for (final Object resultFile : this.qpmeSimulationResultFiles) {
      {
        String experimentName = ((ExperimentModel) resultFile).getExperimentName();
        String qpeResultFile = ((ExperimentModel) resultFile).getModelURI();
        String result = this.parseResult(qpeResultFile, experimentName);
        String line = ((experimentName + " # ") + result);
        String _finalResult = finalResult;
        finalResult = (_finalResult + line);
        InputOutput.<String>println(line);
        this.qpmeSimulationProcessedResults.add(result);
      }
    }
    this.saveOutput(finalResult, ((this.pwdUnix + this.qpmeResultsDirRelative) + finalFileName));
  }
  
  public String parseResult(final String qpeResultFile, final String experimentName) {
    try {
      String _xblockexpression = null;
      {
        String toReturn = "";
        boolean _isFile = new File(qpeResultFile).isFile();
        boolean _equals = (_isFile == false);
        if (_equals) {
          return " File does not exist.";
        }
        HashMap<String, HashMap<String, Double>> summary = new HashMap<String, HashMap<String, Double>>();
        HashMap<String, HashMap<String, LinkedHashMap<String, Double>>> map = this.simQpnParser.parse(qpeResultFile);
        Iterator<Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>> i1 = map.entrySet().iterator();
        while (i1.hasNext()) {
          {
            Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>> _next = i1.next();
            Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>> me = ((Map.Entry<String, HashMap<String, LinkedHashMap<String, Double>>>) _next);
            String _key = me.getKey();
            String placeName = ((String) _key);
            HashMap<String, LinkedHashMap<String, Double>> _value = me.getValue();
            HashMap<String, LinkedHashMap<String, Double>> colorMetricValueHM = ((HashMap<String, LinkedHashMap<String, Double>>) _value);
            Iterator<Map.Entry<String, LinkedHashMap<String, Double>>> i2 = colorMetricValueHM.entrySet().iterator();
            while (i2.hasNext()) {
              {
                Map.Entry<String, LinkedHashMap<String, Double>> _next_1 = i2.next();
                Map.Entry<String, LinkedHashMap<String, Double>> me2 = ((Map.Entry<String, LinkedHashMap<String, Double>>) _next_1);
                String _key_1 = me2.getKey();
                String colorName = ((String) _key_1);
                LinkedHashMap<String, Double> _value_1 = me2.getValue();
                LinkedHashMap<String, Double> metricValueHM = ((LinkedHashMap<String, Double>) _value_1);
                Iterator<Map.Entry<String, Double>> i3 = metricValueHM.entrySet().iterator();
                while (i3.hasNext()) {
                  {
                    Map.Entry<String, Double> _next_2 = i3.next();
                    Map.Entry<String, Double> me3 = ((Map.Entry<String, Double>) _next_2);
                    String _key_2 = me3.getKey();
                    String metricName = ((String) _key_2);
                    Double _value_2 = me3.getValue();
                    double value = (((Double) _value_2)).doubleValue();
                    double tokensPerSec = value;
                    double bytes = this.simQpnParser.parseColorMessageSize(colorName, this.kiloMultipler);
                    double bytesPerSec = (tokensPerSec * bytes);
                    double bitsPerSec = (bytesPerSec * 8);
                    double MbitsPerSec = ((bitsPerSec / 1000) / 1000);
                    String MbitsPerSecTxt = new DecimalFormat("#.##").format(MbitsPerSec);
                    boolean _containsKey = summary.containsKey(placeName);
                    boolean _not = (!_containsKey);
                    if (_not) {
                      HashMap<String, Double> _hashMap = new HashMap<String, Double>();
                      summary.put(placeName, _hashMap);
                    }
                    summary.get(placeName).put(colorName, Double.valueOf(MbitsPerSec));
                  }
                }
              }
            }
          }
        }
        List<String> keyList = IterableExtensions.<String>toList(summary.keySet());
        Collections.<String>sort(keyList);
        for (final String k : keyList) {
          {
            String _return = toReturn;
            String _friendlyName = this.simQpnParser.getFriendlyName(k);
            String _plus = (_friendlyName + " --- ");
            toReturn = (_return + _plus);
            List<String> innerList = IterableExtensions.<String>toList(summary.get(k).keySet());
            Collections.<String>sort(innerList);
            double sum = 0.0;
            for (final String kk : innerList) {
              double _sum = sum;
              Double _get = summary.get(k).get(kk);
              sum = (_sum + (_get).doubleValue());
            }
            String _return_1 = toReturn;
            String _plus_1 = (Double.valueOf(sum) + " Mbps\n");
            toReturn = (_return_1 + _plus_1);
          }
        }
        _xblockexpression = toReturn;
      }
      return _xblockexpression;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  protected void simulateAll() {
    int i = 0;
    for (final Object qpmeFile : this.qpmeFiles) {
      {
        int _i = i;
        i = (_i + 1);
        int _size = this.qpmeFiles.size();
        String _plus = ((("Simulating run #" + Integer.valueOf(i)) + " of ") + Integer.valueOf(_size));
        InputOutput.<String>println(_plus);
        String experimentName = ((ExperimentModel) qpmeFile).getExperimentName();
        String qpeFilePath = ((ExperimentModel) qpmeFile).getModelURI();
        String resultsFilePath = this.simulateSimQPN(qpeFilePath, experimentName);
        InputOutput.<String>println((((("Adding " + experimentName) + " with path ") + qpeFilePath) + " to qpeSimulationResultFiles"));
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, resultsFilePath);
        this.qpmeSimulationResultFiles.add(_experimentModel);
      }
    }
  }
  
  private String simulateSimQPN(final String qpeFile, final String experimentName) {
    try {
      String simulation_output_file = (("/home/mk/workspaces/workspace_DNI_neo/DNI3/DNIinstances/" + experimentName) + ".results.xml");
      simulation_output_file = simulation_output_file.replace("file:/", "/");
      String output_simulation = (" -v /dev/null -o " + simulation_output_file);
      String command = ((this.simqpn + qpeFile) + output_simulation);
      InputOutput.<String>println(("COMMAND: " + command));
      Process p = Runtime.getRuntime().exec(command);
      InputStream _inputStream = p.getInputStream();
      InputStreamReader _inputStreamReader = new InputStreamReader(_inputStream);
      BufferedReader stdInput = new BufferedReader(_inputStreamReader);
      InputStream _errorStream = p.getErrorStream();
      InputStreamReader _inputStreamReader_1 = new InputStreamReader(_errorStream);
      BufferedReader stdError = new BufferedReader(_inputStreamReader_1);
      String s = stdInput.readLine();
      while ((!Objects.equal(s, null))) {
        {
          if ((!this.simulationSilent)) {
            InputOutput.<String>println(s);
          }
          s = stdInput.readLine();
        }
      }
      String se = stdError.readLine();
      while ((!Objects.equal(se, null))) {
        {
          if ((!this.simulationSilent)) {
            InputOutput.<String>println(se);
          }
          se = stdError.readLine();
        }
      }
      return simulation_output_file;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Takes qpme model and returns the contents of the qpe file
   */
  private String modelToTextQpn(final String sourceURI, final String experimentName) {
    return this.stateless_applyEglScript("QPME", sourceURI, "models/qpme.ecore", "egl/dni2qpme.egl", experimentName);
  }
  
  public String stateless_applyEglScript(final String modelName, final String modelURI, final String metaModelFilePath, final String eglScriptPath, final String experimentName) {
    throw new Error("Unresolved compilation problems:"
      + "\nType mismatch: cannot convert from EglTemplateFactoryModuleAdapter to IEolModule"
      + "\nThe method createEmfModel_usingUri(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel");
  }
  
  /**
   * Walks over qpmeModels list and generates text representations of the qpme models
   */
  protected void generateQpeFiles() {
    for (final Object qpnURI : this.qpmeModels) {
      {
        String experimentName = ((ExperimentModel) qpnURI).getExperimentName();
        InputOutput.<String>println(("EXPERIMENT NAME!!!!!!! " + experimentName));
        String modelURI = ((ExperimentModel) qpnURI).getModelURI();
        String contents = this.modelToTextQpn(modelURI, experimentName);
        String outFileName = (experimentName + ".qpe");
        String outputQpnURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
        String unixFilePath = outputQpnURI.replace("file:/", "/");
        this.saveOutput(contents, unixFilePath);
        InputOutput.<String>println(((("Saving model " + outFileName) + "\n to file: ") + unixFilePath));
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, unixFilePath);
        this.qpmeFiles.add(_experimentModel);
      }
    }
  }
  
  /**
   * Runs a eol script over the generated dni models. Gets the models form the dniModels list
   */
  protected void checkDniModels() {
    throw new Error("Unresolved compilation problems:"
      + "\nThe method createEmfModel_usingUri(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel");
  }
  
  /**
   * Takes base_dni_file and generates its variations using the parameters and the eol script
   * The models are stored in the dniModels list
   */
  protected boolean generateDniModel(final String base_dni_file, final List<Variable> eolParameters, final int index, final String experimentName) {
    boolean _xblockexpression = false;
    {
      InputOutput.<String>println((("###### USING BASE MODEL: " + base_dni_file) + "######"));
      ExperimentModel pairExpNameDniModel = null;
      pairExpNameDniModel = this.stateless_applyEolStriptToBaseModel(base_dni_file, "eol/simutools_experiment3.eol", eolParameters, index, experimentName);
      _xblockexpression = this.dniModels.add(pairExpNameDniModel);
    }
    return _xblockexpression;
  }
  
  protected boolean generateDniModel(final String dni_file, final String experimentName) {
    throw new Error("Unresolved compilation problems:"
      + "\nThe method createEmfModel(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel"
      + "\nstore cannot be resolved");
  }
  
  public ExperimentModel stateless_applyEolStriptToBaseModel(final String base_dni_file, final String eolScriptPath, final List<Variable> eolParameters, final int index, final String experimentName) {
    throw new Error("Unresolved compilation problems:"
      + "\nThe method createEmfModel(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel"
      + "\nstore cannot be resolved");
  }
  
  /**
   * Processes the dniModels list and generate a qpn representation for each. Qpn models are saved to the qpmeModels list
   */
  protected void transformAll() {
    for (final Object dniURI : this.dniModels) {
      {
        String experimentName = ((ExperimentModel) dniURI).getExperimentName();
        String modelURI = ((ExperimentModel) dniURI).getModelURI();
        String qpmeResultURI = this.transformDniToQpn(modelURI, experimentName);
        ExperimentModel _experimentModel = new ExperimentModel(experimentName, qpmeResultURI);
        this.qpmeModels.add(_experimentModel);
      }
    }
  }
  
  /**
   * Processes a single dni model and returns uri to the qpn representation
   */
  private String transformDniToQpn(final String sourceURI, final String experimentName) {
    List<Variable> origParameterList = this.epsilonTransformationParameters.get(experimentName);
    String outFileName = (experimentName + ".qpmemodel");
    String destinationModelURI = ((this.pwdURI + this.qpmeModelsDirRelative) + outFileName);
    String outputQpnURI = this.stateless_applyEtlStript("DNI", sourceURI, "metamodels/dnimm3.ecore", 
      "QPME", destinationModelURI, "metamodels/qpme.ecore", 
      "etl/standalone-DNI-QPN.etl", origParameterList, experimentName);
    return outputQpnURI;
  }
  
  public String stateless_applyEtlStript(final String sourceModelName, final String sourceModelURI, final String sourceMetaModelURI, final String destinationModelName, final String destinationModelURI, final String destinationMetaModelURI, final String etlScriptPath, final List<Variable> eolParameters, final String experimentName) {
    throw new Error("Unresolved compilation problems:"
      + "\nType mismatch: cannot convert from EtlModule to IEolModule"
      + "\nThe method createEmfModel_usingUri(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel"
      + "\nThe method createEmfModel_usingUri(String, String, String, boolean, boolean) from the type DNIRunner refers to the missing type EmfModel"
      + "\nstore cannot be resolved");
  }
  
  protected void cleanup() {
    String resultsDir = (this.pwdUnix + this.qpmeResultsDirRelative);
    List<String> existingResultsFileList = this.getFileList(resultsDir, ".results.xml", "");
    List<String> flatHQPNfiles = this.getFileList(this.pwdUnix, ".xml", "SimQPN_FlatHQPN_");
    List<String> toDelete = CollectionLiterals.<String>newLinkedList();
    toDelete.addAll(existingResultsFileList);
    toDelete.addAll(flatHQPNfiles);
    for (final String fileToDelete : existingResultsFileList) {
      {
        boolean deleted = this.deleteFile(fileToDelete);
        if (deleted) {
          InputOutput.<String>println(("Deleted file " + fileToDelete));
        } else {
          InputOutput.<String>println(("Could not delete file " + fileToDelete));
        }
      }
    }
  }
  
  /**
   * Executes Epsilon code over models
   */
  private Object prepareAndExecute(final IEolModule module, final String source, final List<IModel> models, final List<Variable> parameters) throws Exception {
    URI resURI = this.getFileURI(source);
    module.parse(resURI);
    int _size = module.getParseProblems().size();
    boolean _greaterThan = (_size > 0);
    if (_greaterThan) {
      System.err.println("Parse errors occured...");
      List<ParseProblem> _parseProblems = module.getParseProblems();
      for (final ParseProblem problem : _parseProblems) {
        System.err.println(problem.toString());
      }
      return null;
    }
    for (final IModel model : models) {
      module.getContext().getModelRepository().addModel(model);
    }
    for (final Variable parameter : parameters) {
      module.getContext().getFrameStack().put(parameter);
    }
    Object result = this.execute(module);
    for (final IModel model_1 : models) {
      module.getContext().getModelRepository().removeModel(model_1);
    }
    return result;
  }
  
  protected Object execute(final IEolModule module) throws EolRuntimeException {
    return module.execute();
  }
  
  /**
   * Creates emf model object if a file addressed over uri exists. Uses file-based META-model
   */
  protected /* EmfModel */Object createEmfModel(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    throw new Error("Unresolved compilation problems:"
      + "\nEmfModel cannot be resolved to a type."
      + "\nEmfModel cannot be resolved."
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nPROPERTY_NAME cannot be resolved"
      + "\nPROPERTY_FILE_BASED_METAMODEL_URI cannot be resolved"
      + "\nPROPERTY_MODEL_URI cannot be resolved"
      + "\nPROPERTY_READONLOAD cannot be resolved"
      + "\nPROPERTY_STOREONDISPOSAL cannot be resolved"
      + "\nload cannot be resolved");
  }
  
  /**
   * Creates emf model object if a file addressed over uri does not exist
   */
  protected /* EmfModel */Object createEmfModel_usingUri(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    throw new Error("Unresolved compilation problems:"
      + "\nEmfModel cannot be resolved to a type."
      + "\nEmfModel cannot be resolved."
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nPROPERTY_NAME cannot be resolved"
      + "\nPROPERTY_FILE_BASED_METAMODEL_URI cannot be resolved"
      + "\nPROPERTY_MODEL_URI cannot be resolved"
      + "\nPROPERTY_READONLOAD cannot be resolved"
      + "\nPROPERTY_STOREONDISPOSAL cannot be resolved"
      + "\nload cannot be resolved");
  }
  
  /**
   * The same as createEmfModel but allows addressing of META-model using uri
   */
  protected /* EmfModel */Object createEmfModelByURI(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    throw new Error("Unresolved compilation problems:"
      + "\nEmfModel cannot be resolved to a type."
      + "\nEmfModel cannot be resolved."
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nThe method or field EmfModel is undefined"
      + "\nPROPERTY_NAME cannot be resolved"
      + "\nPROPERTY_METAMODEL_URI cannot be resolved"
      + "\nPROPERTY_MODEL_URI cannot be resolved"
      + "\nPROPERTY_READONLOAD cannot be resolved"
      + "\nPROPERTY_STOREONDISPOSAL cannot be resolved"
      + "\nload cannot be resolved");
  }
  
  private URI getFileURI(final String fileName) throws URISyntaxException {
    URL res = DNIRunner.class.getResource(fileName);
    URI binUri = res.toURI();
    URI uri = null;
    int _indexOf = binUri.toString().indexOf("bin");
    boolean _greaterThan = (_indexOf > (-1));
    if (_greaterThan) {
      String _replaceAll = binUri.toString().replaceAll("bin", "src");
      URI _uRI = new URI(_replaceAll);
      uri = _uRI;
    } else {
      uri = binUri;
    }
    return uri;
  }
  
  public void saveOutput(final String txt, final String fname) {
    try {
      File file = new File(fname);
      file.createNewFile();
      FileWriter writer = new FileWriter(file);
      writer.write(txt);
      writer.flush();
      writer.close();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
